/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : DateListModel
   date       : 05.01.98
   description: a model for list of current loaded dates
   modified   : 15.02.98 - periodic date list added
                02.03.98 - bug in isOverlapping removed (empty vector)
                03.03.98 - method getUsedTimeVector added
                11.03.98 - detection of deleted instances and
                           overlapping entries added

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class DateListModel extends Observable {

  // instance variable declarations
  private Vector vector;
  private Vector periodicVector;
  private Vector overlapVector;
  private boolean invalidFlag;
  private boolean periodicInvalidFlag;
  private boolean newFlag;

  // constructors
  public DateListModel() {
    vector              = new Vector();
    periodicVector      = new Vector();
    overlapVector       = new Vector();
    invalidFlag         = false;
    periodicInvalidFlag = false;
    newFlag             = false;
  } // end constructor DateListModel

  // methods
  public void removeAllCalendarEntries() {
    vector.removeAllElements();
  } // end removeAllCalendarEntries

  public void removeAllPeriodicEntries() {
    periodicVector.removeAllElements();
  } // end removeAllPeriodicEntries

  public void addCalendarEntry(CalendarEntry entry) {
    // insert sorted
    CalendarEntry t;
    int n = vector.size();
    int i = 0;

    if (entry.isDeletedInstance()) {
      boolean done = false;

      while ((i < n) && !done) {
        if (entry.deletedInstanceMatches((CalendarEntry)vector.elementAt(i)))
          done = true;
        else
          i++;
        // end if
      } // end while
      if (done) vector.removeElementAt(i);
    }
    else {
      while (i < n) {
        t = (CalendarEntry)vector.elementAt(i);
        if (entry.getStart().after(t.getStart())) i++;
          else n = 0;
      } // end while
      vector.insertElementAt(entry,i);
    } // end if
  } // end addCalendarEntry

  public void addPeriodicEntry(CalendarEntry entry) {

    periodicVector.addElement(entry);
  } // end addPeriodicEntry

  public void done() {
    int i;
    int n = vector.size()-1;
    CalendarEntry c1,c2;

    overlapVector.removeAllElements();
    for (i=0; i<n; i++) {
      c1 = (CalendarEntry)vector.elementAt(i);
      c2 = (CalendarEntry)vector.elementAt(i+1);
      if (c1.getEnd().after(c2.getStart())) {
        overlapVector.addElement(c1);
        overlapVector.addElement(c2);
      } // end if
    } // end for
    invalidFlag         = false;
    periodicInvalidFlag = false;
    newFlag             = true;
    notifyNow();
  } // end done;

  public void invalidate() {
    invalidFlag         = true;
    newFlag             = false;
    notifyNow();
  } // end invalidate;

  public void periodicInvalidate() {
    periodicInvalidFlag = true;
    newFlag             = false;
    notifyNow();
  } // end periodicInvalidate;

  public boolean isNew() {
    return newFlag;
  } // end isNew

  public boolean isInvalid() {
    return invalidFlag;
  } // end isInvalid

  public boolean isPeriodicInvalid() {
    return periodicInvalidFlag;
  } // end isPeriodicInvalid

  public Enumeration getCalendarEntries() {
    return vector.elements();
  } // end getCalendarEntries

  public Enumeration getPeriodicEntries() {
    return periodicVector.elements();
  } // end getPeriodicEntries

  public Enumeration getOverlappingEntries() {
    return overlapVector.elements();
  } // end getOverlapingEntries

  public boolean hasOverlappingEntries() {
    return (overlapVector.size() > 1);
  } // end hasOverlappingEntries

  public boolean isOverlapping(CalendarEntry entry) {
    // returns true if entry is overlapping a date in the list of the model
    if (vector.size() == 0) return false;
    Vector curVector = new Vector();
    DateDupel d1 = new DateDupel();
    DateDupel d2 = new DateDupel();
    int i,n;
    int l,r,x;

    if (entry.isPeriodic()) {
      d1.start.setTime(((CalendarEntry)vector.firstElement()).getStart().getTime());
      d1.end.setTime(((CalendarEntry)vector.lastElement()).getEnd().getTime());
      Enumeration enum = entry.makePeriodicInstances(d1.start,d1.end);

      while (enum.hasMoreElements()) curVector.addElement(enum.nextElement());
    }
    else {
      curVector.addElement(entry);
    } // end if
    n = curVector.size();
    for (i=0; i<n; i++) {
      l = 0; r = vector.size()-1;
      d1.start.setTime(((CalendarEntry)curVector.elementAt(i)).getStart().getTime());
      d1.end.setTime(((CalendarEntry)curVector.elementAt(i)).getEnd().getTime());
      do {
        x = (l + r) / 2;
        d2.start.setTime(((CalendarEntry)vector.elementAt(x)).getStart().getTime());
        d2.end.setTime(((CalendarEntry)vector.elementAt(x)).getEnd().getTime());
        if (d1.before(d2)) r = x-1; else l = x+1;
      } while (!(d1.overlaps(d2) || (l > r)));
      if (d1.overlaps(d2)) return true;
    } // end for
    return false;
  } // end isOverlapping

  private void notifyNow() {
    Assert.out.print("DateListModell: entries ");
    if (isInvalid()) Assert.out.print("in");
    Assert.out.print("valid, periodic ");
    if (isPeriodicInvalid()) Assert.out.print("in");
    Assert.out.print("valid, date list ");
    if (isNew()) Assert.out.println("new.");
      else Assert.out.println("not changed.");
    setChanged();
    notifyObservers();
  } // end notifyNow

  public Vector getUsedTimeVector() {
    Vector curVector = new Vector();
    int i,n;
    CalendarEntry entry;
    DateDupel dupel;
    DateDupel lastDupel = new DateDupel();

    n = vector.size();
    for (i=0; i<n; i++) {
      entry = (CalendarEntry)vector.elementAt(i);
      dupel = new DateDupel(entry.getStart().getTime(),
                            entry.getEnd().getTime());
      if (curVector.size() > 0) {
        lastDupel = (DateDupel)curVector.lastElement();
        if (dupel.startsInOrExtends(lastDupel))
          lastDupel.extendEnd(dupel);
        else
          curVector.addElement(dupel);
        // end if
      }
      else {
        curVector.addElement(dupel);
      } // end if
    } // end for
    return curVector;
  } // end getUsedTimeVector

} // end DateListModel

/* ******************************************************************
   end of file
*********************************************************************/
