/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : DateListRefresher
   date       : 01.01.98
   description: this class observes the scope model and refreshes
                the date list model, if nessesary
   modified   : 15.02.98 - update of periodic entries added

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class DateListRefresher implements Observer {

  // instance variable declarations
  private LoginInfoModel lModel;
  private ScopeModel sModel;
  private DateListModel dModel;
  private CalendarDatabase db;
  private String errorMsg;
  private String[] userList;

  // constructors
  public DateListRefresher(LoginInfoModel lModel, ScopeModel sModel,
                           DateListModel dModel,
                           CalendarDatabase db) {
    this.lModel = lModel;
    this.sModel = sModel;
    this.dModel = dModel;
    this.db     = db;
    errorMsg    = null;
    userList    = new String[1];
    sModel.addObserver(this);
    dModel.addObserver(this);
  } // end constructor DateListRefresher

  // methods
  public void update(Observable obs, Object arg) {

    // if not loged in, do nothing
    //if (!lModel.getLogedIn()) return;

    boolean sModelInvoke   = obs.equals(sModel);
    boolean dModelInvoke   = obs.equals(dModel);
    boolean periodicUpdate = (sModelInvoke && sModel.userChanged()) ||
                             (dModelInvoke && dModel.isPeriodicInvalid());
    boolean normalUpdate   = (periodicUpdate) ||
                             (sModelInvoke && sModel.scopeChanged()) ||
                             (dModelInvoke && dModel.isInvalid());

    Assert.out.println("DateListRefresher: "+
    "date "+(sModel.dateChanged()?"changed":"n.c.")+
    ", scope "+(sModel.scopeChanged()?"changed":"n.c.")+
    ", user "+(sModel.userChanged()?"changed":"n.c.")+
    ", selection "+(sModel.selObChanged()?"changed":"n.c."));
    if (normalUpdate) {
      if (periodicUpdate)
        Assert.out.print("DateListRefresher: periodic update invoked by ");
      else
        Assert.out.print("DateListRefresher: normal update invoked by ");
      // end if
      if (sModelInvoke) Assert.out.println("ScopeModel.");
        else Assert.out.println("DateListModel.");
      Date start  = sModel.getStartOfScope();
      Date end    = sModel.getEndOfScope();
      userList[0] = sModel.getUser();
      if (lModel.getLogedIn()) {
        buildUp(periodicUpdate,dModel,start,end,userList);
      }
      else {
        dModel.removeAllPeriodicEntries();
        dModel.removeAllCalendarEntries();
        dModel.done();
      } // end if
    } // end if
  } // end update

  private void buildUp(boolean periodic, DateListModel model,
                       Date start, Date end,String[] userList) {
    Enumeration enum,enum2;
    CalendarEntry entry;
    int n = userList.length;
    int i;

    if (periodic) {
      model.removeAllPeriodicEntries();
      errorMsg = null;
      i        = 0;
      while ((i < n) && (errorMsg == null)) {
        enum     = db.getDates(userList[i],start,end,true);
        errorMsg = db.getErrorMsg();
        while ((errorMsg == null) && enum.hasMoreElements()) {
          model.addPeriodicEntry((CalendarEntry)enum.nextElement());
          errorMsg = db.getErrorMsg();
        } // end while
        i++;
      } // end while
    }
    else {
      errorMsg = null;
    } // end if
    model.removeAllCalendarEntries();
    enum = model.getPeriodicEntries();
    while (enum.hasMoreElements()) {
      entry = (CalendarEntry)enum.nextElement();
      enum2 = entry.makePeriodicInstances(start,end);
      while (enum2.hasMoreElements()) {
        model.addCalendarEntry((CalendarEntry)enum2.nextElement());
      } // end while
    } // end while
    // read non periodic dates
    i = 0;
    while ((i < n) && (errorMsg == null)) {
      enum = db.getDates(userList[i],start,end,false);
      errorMsg = db.getErrorMsg();
      while (enum.hasMoreElements() && (errorMsg == null)) {
        model.addCalendarEntry((CalendarEntry)enum.nextElement());
        errorMsg = db.getErrorMsg();
      } // end while
      i++;
    } // end while
    if (errorMsg != null)
      Assert.out.println("DateListRefresher: update failed.");
    model.done();
  } // end buildUp

  private void clipOut(DateDupel dateDupel, Vector vector) {
    boolean found = true;
    int l,r,x,n;
    DateDupel curDupel;

    do {
      n = vector.size();
      if (n < 1) break;
      l = 0; r = n-1;
      do {
        x = (l + r) / 2;
        curDupel = (DateDupel)vector.elementAt(x);
        if (dateDupel.before(curDupel)) r = x-1; else l = x+1;
      } while (!(dateDupel.overlaps(curDupel) || (l > r)));
      if (dateDupel.overlaps(curDupel)) {
        if (curDupel.isIn(dateDupel)) {
          vector.removeElementAt(x);
        } else if (dateDupel.startInAfterAndEndsInBefore(curDupel)) {
          vector.insertElementAt(new DateDupel(dateDupel.end.getTime(),
                                   curDupel.end.getTime()),x+1);
          curDupel.end.setTime(dateDupel.start.getTime());
        } else if (dateDupel.startsIn(curDupel)) {
          curDupel.end.setTime(dateDupel.start.getTime());
        } else { // if curDupel only overlaps with dateDupel
          curDupel.clip(dateDupel);
        } // end if
      }
      else {
        found = false;
      } // end if
    } while (found);
  } // end clipOut

  public void clipFreeDateVector(Vector vector, String[] persons,
                                 int hour, int min) {
    if (vector.size() == 0) return;
    Vector usedVector;
    Date start = new Date(((DateDupel)vector.firstElement()).start.getTime());
    Date end   = new Date(((DateDupel)vector.lastElement()).end.getTime());
    DateListModel tmpModel = new DateListModel();
    Calendar calendar      = CalendarCreator.get();
    DateDupel dateDupel;
    int i,n;

    buildUp(true,tmpModel,start,end,persons);
    usedVector = tmpModel.getUsedTimeVector();
    n          = usedVector.size();
    i          = 0;
    while ((i < n) && (vector.size() > 0)) {
      dateDupel = (DateDupel)usedVector.elementAt(i);
      clipOut(dateDupel,vector);
      i++;
    } // end while
    // remove all dupels which are to short
    n = vector.size();
    i = 0;
    while (i < n) {
      dateDupel = (DateDupel)vector.elementAt(i);
      calendar.setTime(dateDupel.start);
      calendar.add(Calendar.HOUR_OF_DAY,hour);
      calendar.add(Calendar.MINUTE,min);
      if (calendar.getTime().after(dateDupel.end)) {
        vector.removeElementAt(i);
        n--;
      }
      else {
        i++;
      } // end if
    } // end while
  } // end clipFreeDateVector

  public String getErrorMsg() {
    String t = errorMsg;

    errorMsg = null;
    return t;
  } // end getErrorMsg

} // end DateListRefresher

/* ******************************************************************
   end of file
*********************************************************************/
